"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SecretProvider = exports.GitHubAppTokenProvider = exports.FileSecretProvider = exports.configureAndListen = void 0;
const fastify_1 = require("fastify");
const validate_token_1 = require("./oidc/validate-token");
const FileProvider_1 = require("./providers/FileProvider");
Object.defineProperty(exports, "FileSecretProvider", { enumerable: true, get: function () { return FileProvider_1.FileSecretProvider; } });
const GitHubAppProvider_1 = require("./providers/GitHubAppProvider");
Object.defineProperty(exports, "GitHubAppTokenProvider", { enumerable: true, get: function () { return GitHubAppProvider_1.GitHubAppTokenProvider; } });
const SecretProvider_1 = require("./SecretProvider");
Object.defineProperty(exports, "SecretProvider", { enumerable: true, get: function () { return SecretProvider_1.SecretProvider; } });
const DEFAULT_PORT = process.env.PORT ? parseInt(process.env.PORT, 10) : 8080;
const configureAndListen = async (config, port = DEFAULT_PORT) => {
    const fastify = (0, fastify_1.default)({
        logger: true,
    });
    fastify.get('/healthcheck', async () => {
        return { alive: true };
    });
    fastify.post('/exchange', {
        schema: {
            body: {
                type: 'object',
                properties: {
                    token: { type: 'string' },
                },
            },
        },
    }, async (req, reply) => {
        req.log.info('Received secret exchange request');
        const { token } = req.body;
        let validatedClaims = null;
        let secretProviders = null;
        for (const configuration of config) {
            validatedClaims = await (0, validate_token_1.getValidatedToken)(token, `https://oidc.circleci.com/org/fbdaa81e-9fcd-4b31-9f5d-11feaa8ef52d${configuration.organizationId}`);
            if (validatedClaims) {
                req.log.info(`Validated incoming OIDC token from: ${validatedClaims.sub}`);
                secretProviders = configuration.secrets;
                break;
            }
        }
        if (!validatedClaims || !secretProviders) {
            req.log.warn('Failed to validing incoming OIDC token, did not match any configured organization');
            return reply.code(401);
        }
        // Do a quick-pass to filter down secret providers based on contextId and projectId claims
        const projectId = validatedClaims['oidc.circleci.com/project-id'];
        // It is currently specified by CircleCI that this token will only ever have a single context ID
        // Ref: https://circleci.com/docs/openid-connect-tokens/#format-of-the-openid-connect-id-token
        const contextId = validatedClaims['oidc.circleci.com/context-ids'][0];
        const filteredSecretProviders = secretProviders.filter((provider) => {
            if (!provider.filters.projectIds.includes(projectId))
                return false;
            if (!provider.filters.contextIds.includes(contextId))
                return false;
            return true;
        });
        const secretProvidersInitialized = filteredSecretProviders.map((provider) => provider.provider(projectId, contextId));
        // Some secret providers need to load content, and some secrets use the same
        // loaded content.  To reduce work we share loadable content across secret providers
        const contentToLoad = {};
        // Get content we need to load and share
        const loaders = [];
        for (const secretProvider of secretProvidersInitialized) {
            const key = secretProvider.loadableContentKey();
            if (contentToLoad[key])
                continue;
            const promise = secretProvider.loadContent();
            contentToLoad[key] = promise;
            loaders.push(promise);
        }
        // Wait for all content to load
        await Promise.all(loaders);
        const secretsToSend = {};
        await Promise.all(secretProvidersInitialized.map(async (provider) => {
            const key = provider.loadableContentKey();
            const providedSecrets = await provider.provideSecrets(await contentToLoad[key]);
            for (const secretKey of Object.keys(providedSecrets)) {
                if (secretsToSend[secretKey]) {
                    throw new Error(`Two secret providers provided the same secret key: "${secretKey}"`);
                }
                secretsToSend[secretKey] = providedSecrets[secretKey];
            }
        }));
        req.log.info(`Respond to a secrets exchange request from "${validatedClaims.sub}" with ${JSON.stringify(Object.keys(secretsToSend))}`);
        return secretsToSend;
    });
    await fastify.listen({
        port,
    });
};
exports.configureAndListen = configureAndListen;
//# sourceMappingURL=index.js.map