"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getValidatedToken = void 0;
const got_1 = require("got");
const openid_client_1 = require("openid-client");
const jwkToPem = require("jwk-to-pem");
const jwt = require("jsonwebtoken");
const path = require("path");
async function getValidatedToken(token, discoveryUrl) {
    var _a;
    if (!token) {
        // Token is an empty string, no point even trying
        return null;
    }
    let issuer;
    try {
        issuer = await openid_client_1.Issuer.discover(discoveryUrl);
    }
    catch (_b) {
        // Failed to discover the OIDC issuer
        return null;
    }
    if (!((_a = issuer === null || issuer === void 0 ? void 0 : issuer.metadata) === null || _a === void 0 ? void 0 : _a.jwks_uri)) {
        // Project is not eligible for JWKS backed OIDC credential exchange
        return null;
    }
    const jwks = await got_1.default.get(issuer.metadata.jwks_uri, {
        throwHttpErrors: false,
    });
    if (jwks.statusCode !== 200) {
        // Project is not eligible for JWKS backed OIDC credential exchange
        // Failed to load JWKS
        return null;
    }
    // Full claims, not validated
    const claims = jwt.decode(token, { complete: true });
    if (!claims) {
        // Failed to decode token, invalid token provided
        return null;
    }
    const { keys } = JSON.parse(jwks.body);
    // Ensure keys is present, an array, and all items have a kid for matching
    if (!keys || !Array.isArray(keys) || !keys.every((key) => !!key.kid)) {
        // Invalid JWKS endpoints
        return null;
    }
    const nonNullClaims = claims;
    const key = keys.find((key) => key.kid && key.kid === nonNullClaims.header.kid);
    if (!key) {
        // Invalid kid found in the token provided
        return null;
    }
    // Try convert the discovered key into a PEM for jwt verification
    const pem = jwkToPem(key);
    let verifiedClaims;
    try {
        verifiedClaims = jwt.verify(token, pem, {
            complete: true,
            algorithms: ['RS256'],
            issuer: discoveryUrl,
            audience: path.basename(discoveryUrl),
        });
    }
    catch (err) {
        // Could not verify the provided token against the OIDC provider
        return null;
    }
    return verifiedClaims.payload;
}
exports.getValidatedToken = getValidatedToken;
//# sourceMappingURL=validate-token.js.map